from machine import Pin
from time import sleep
from machine import Pin, I2C
from vl53l0x import VL53L0X

#extra variables
step = 0
sodas = 0
fill_cup = 10
button1_hit = 0
button2_hit = 0
button3_hit = 0
button4_hit = 0

#setting up time of flight sensor
sda = Pin(0)
scl = Pin(1)
id = 0
i2c = I2C(id=id, sda=sda, scl=scl)
tof = VL53L0X(i2c)
budget = tof.measurement_timing_budget_us
print("Budget was:", budget)
tof.set_measurement_timing_budget(40000)
tof.set_Vcsel_pulse_period(tof.vcsel_period_type[0], 12)
tof.set_Vcsel_pulse_period(tof.vcsel_period_type[1], 8)

#setting up buttons
# Button GPIO Pins
button1_gpio = 18 # start
button2_gpio = 19 # lower
button3_gpio = 21  # middle
button4_gpio = 20  # upper
# Button Pins
button1 = Pin(button1_gpio, Pin.IN, Pin.PULL_UP)
button2 = Pin(button2_gpio, Pin.IN, Pin.PULL_UP)
button3 = Pin(button3_gpio, Pin.IN, Pin.PULL_UP)
button4 = Pin(button4_gpio, Pin.IN, Pin.PULL_UP)
# Variables to track button states
button1_state = 1
button2_state = 1
button3_state = 1
button4_state = 1

#setting up limit action switch
# Limit Action Switch GPIO Pin
limit_switch_gpio = 15
#limit pin
limit_switch = Pin(limit_switch_gpio, Pin.IN, Pin.PULL_DOWN)

#setting up stepper motor
#pins
IN1 = Pin(2, Pin.OUT)
IN2 = Pin(3, Pin.OUT)
IN3 = Pin(4, Pin.OUT)
IN4 = Pin(5, Pin.OUT)
#pin array
pins = [IN1, IN2, IN3, IN4]
#turning sequence for pins
sequence_clockwise = [[1, 0, 0, 0], [0, 1, 0, 0], [0, 0, 1, 0], [0, 0, 0, 1]]
sequence_counterclockwise = [[0, 0, 0, 1], [0, 0, 1, 0], [0, 1, 0, 0], [1, 0, 0, 0]]
#stepper paramiters
steps_per_revolution = 200 
delay = 0.001
# Set all pins to LOW initially
for pin in pins:
    pin.value(0)
    
#pumps
#two
motor2_in1 = machine.Pin(6, machine.Pin.OUT)
motor2_in2 = machine.Pin(7, machine.Pin.OUT)
#three
motor3_in1 = machine.Pin(8, machine.Pin.OUT)
motor3_in2 = machine.Pin(9, machine.Pin.OUT)
#one
motor1_in1 = machine.Pin(12, machine.Pin.OUT)
motor1_in2 = machine.Pin(13, machine.Pin.OUT)

def step_forward(num_steps):
    sequence = sequence_clockwise
    for _ in range(num_steps):
        for step in sequence:
            for i in range(len(pins)):
                pins[i].value(step[i])
            sleep(delay)
            
def step_backward(num_steps):
    sequence = sequence_counterclockwise
    for _ in range(num_steps):
        for step in sequence:
            for i in range(len(pins)):
                pins[i].value(step[i])
            sleep(delay)

def stepper_idle():
    for pin in pins:
        pin.value(0)
        
def button_callback(pin):
    global button1_state, button2_state, button3_state, button4_state, button1_hit, button2_hit, button3_hit, button4_hit

    if pin == button1 and button1_state == 0:
        if button1_hit == 0:
            button1_hit = 1
            sleep(.1)
        elif button1_hit == 1:
            button1_hit = 0
            sleep(.1)
        sleep(.1)
            
    elif pin == button2 and button2_state == 0:
        if button2_hit == 0:
            button2_hit = 1
            sleep(.1)
        elif button2_hit == 1:
            button2_hit = 0
            sleep(.1)
    elif pin == button3 and button3_state == 0:
        if button3_hit == 0:
            button3_hit = 1
            sleep(.125)
        elif button3_hit == 1:
            button3_hit = 0
            sleep(.125)
    elif pin == button4 and button4_state == 0:
        if button4_hit == 0:
            button4_hit = 1
            sleep(.125)
        elif button4_hit == 1:
            button4_hit = 0
            sleep(.125)
    button1_state = button1.value()
    button2_state = button2.value()
    button3_state = button3.value()
    button4_state = button4.value()
    
button1.irq(trigger=Pin.IRQ_FALLING | Pin.IRQ_RISING, handler=button_callback)
button2.irq(trigger=Pin.IRQ_FALLING | Pin.IRQ_RISING, handler=button_callback)
button3.irq(trigger=Pin.IRQ_FALLING | Pin.IRQ_RISING, handler=button_callback)
button4.irq(trigger=Pin.IRQ_FALLING | Pin.IRQ_RISING, handler=button_callback)

def zero_machine():
    while limit_switch.value() == 0:
        step_backward(1)
    
def move_to_place(place):
    print("move to place was called")
    while True:
        distance = tof.ping() - 50
        if distance < place:
            step_forward(1)
        elif distance > place:
            step_backward(1)
        else:
            break
        sleep(delay)  # Add a small delay between steps for smoother motion

    stepper_idle()

while True:
    
    button1_state = button1.value()
    button2_state = button2.value()
    button3_state = button3.value()
    button4_state = button4.value()
    #zero machine when turned on
    if step == 0:
        zero_machine()
        step = 1
        sodas = 0
    elif step == 1:
        if button1_hit == 1:
            if button2_hit == 1:
                sodas += 1
            elif button3_hit == 1:
                sodas += 1
            elif button4_hit == 1:
                sodas += 1
            step = 2
    elif step == 2:
        if button2_hit == 1:
            move_to_place(136)
            motor1_in1.on()
            motor1_in2.off()
            sleep(fill_cup/sodas)
            motor1_in1.off()
            motor1_in2.off()
        if button3_hit == 1:
            move_to_place(236)
            motor2_in1.on()
            motor2_in2.off()
            sleep(fill_cup/sodas)
            motor2_in1.off()
            motor2_in2.off()
        if button4_hit == 1:
            move_to_place(330)
            motor3_in1.on()
            motor3_in2.off()
            sleep(fill_cup/sodas)
            motor3_in1.off()
            motor3_in2.off()
        step = 3
    elif step == 3:
        zero_machine()
        sleep(5)
        step = 0
        button1_hit = 0
        button2_hit = 0
        button3_hit = 0
        button4_hit = 0
    
     # Perform other tasks or pause the program
    sleep(0.125)

    # Check if all buttons are released
    if button1_state == 1 and button2_state == 1 and button3_state == 1 and button4_state == 1:
        stepper_idle()
    print(button1_hit)
    print(button2_hit)
    print(button3_hit)
    print(button4_hit)
    print()
    
        
    
            
            
        
        
        
    


