from machine import Pin
from time import sleep

IN1 = Pin(2, Pin.OUT)
IN2 = Pin(3, Pin.OUT)
IN3 = Pin(4, Pin.OUT)
IN4 = Pin(5, Pin.OUT)

pins = [IN1, IN2, IN3, IN4]

sequence_clockwise = [[1, 0, 0, 0], [0, 1, 0, 0], [0, 0, 1, 0], [0, 0, 0, 1]]
sequence_counterclockwise = [[0, 0, 0, 1], [0, 0, 1, 0], [0, 1, 0, 0], [1, 0, 0, 0]]

steps_per_revolution = 200  # Adjust this based on your motor's specifications
delay = 0.001  # Delay between steps in seconds

# Set all pins to LOW initially
for pin in pins:
    pin.value(0)

# Button GPIO Pins
button1_gpio = 21  # start
button2_gpio = 18 # lower
button3_gpio = 20  # middle
button4_gpio = 19  # upper

# Limit Action Switch GPIO Pin
limit_switch_gpio = 15

# Button Pins
button1 = Pin(button1_gpio, Pin.IN, Pin.PULL_UP)
button2 = Pin(button2_gpio, Pin.IN, Pin.PULL_UP)
button3 = Pin(button3_gpio, Pin.IN, Pin.PULL_UP)
button4 = Pin(button4_gpio, Pin.IN, Pin.PULL_UP)

# Limit Action Switch Pin
limit_switch = Pin(limit_switch_gpio, Pin.IN, Pin.PULL_DOWN)

# Variables to track button states
button1_state = 1
button2_state = 1
button3_state = 1
button4_state = 1

def step_forward(num_steps):
    sequence = sequence_counterclockwise
    for _ in range(num_steps):
        for step in sequence:
            for i in range(len(pins)):
                pins[i].value(step[i])
            sleep(delay)

def step_backward(num_steps):
    sequence = sequence_clockwise
    for _ in range(num_steps):
        for step in sequence:
            for i in range(len(pins)):
                pins[i].value(step[i])
            sleep(delay)

def stepper_idle():
    for pin in pins:
        pin.value(0)

def zero_machine():
    while limit_switch.value() == 0:
        step_backward(1)
        
    stepper_idle()

# Callback function for button press
def button_callback(pin):
    global button1_state, button2_state, button3_state, button4_state

    if pin == button1 and button1_state == 0:
        zero_machine()

    elif pin == button2 and button2_state == 0:
        step_forward(100)

    elif pin == button3 and button3_state == 0:
        step_forward(350)

    elif pin == button4 and button4_state == 0:
        step_forward(550)

    # Update button states
    button1_state = button1.value()
    button2_state = button2.value()
    button3_state = button3.value()
    button4_state = button4.value()

# Configure interrupt handlers for button press
button1.irq(trigger=Pin.IRQ_FALLING | Pin.IRQ_RISING, handler=button_callback)
button2.irq(trigger=Pin.IRQ_FALLING | Pin.IRQ_RISING, handler=button_callback)
button3.irq(trigger=Pin.IRQ_FALLING | Pin.IRQ_RISING, handler=button_callback)
button4.irq(trigger=Pin.IRQ_FALLING | Pin.IRQ_RISING, handler=button_callback)

# Main loop
while True:
    # Update button states
    button1_state = button1.value()
    button2_state = button2.value()
    button3_state = button3.value()
    button4_state = button4.value()

    # Perform other tasks or pause the program
    sleep(0.01)

    # Check if all buttons are released
    if button1_state == 1 and button2_state == 1 and button3_state == 1 and button4_state == 1:
        stepper_idle()
